use Cwd;
use strict;
use vars qw /$CONFIG $OS/;
delete $ENV{SERVER_NAME};
delete $ENV{PATH_INFO};

BEGIN
{
    $CONFIG = {};
    
    open FP, "<install.conf" or die "Cannot read-open install.conf";
    while (my $string = <FP>)
    {
	chomp ($string);

	# if the line starts with a comment, then it's all a comment
	next if ($string =~ /^\#.*/);
	
        # has this line got comments?
        if ($string =~ /(?<!\\)\#/)
        {
            my ($key, $val, $cmt) = $string =~ /(.*?)\s+(.*?)\s+(.*)/;
            $val =~ s/\s+$//;
	    $CONFIG->{$key} = $val;
        }
        else
        {
            if ($string =~ /\S+(\s+)\S+/)
            {
                my ($key, $val) = $string =~ /(.*?)\s+(.*)/;
                $val =~ s/\s+$//;
		$CONFIG->{$key} = $val;
            }
            else
            {
                my $key = $string;
                my $val = 1;
		$CONFIG->{$key} = $val;
            }
        }
    }
    close FP;
   
    $CONFIG->{MKDOC_DIR} = shift (@ARGV) || die "Usage: $0 <mkdoc_dir>"; 
    # unshift @INC to add MKDoc install dir
    unshift @INC, $CONFIG->{MKDOC_DIR};
}

eval
{
    use lib::Config;
    use lib::Template;
    use lib::Exception;
    use lib::sql::Category;
    use lib::Config;
    use lib::sql::type::ALL;
    use Data::Dumper;
};
die $@ if (defined $@ and $@);

eval { main() };
if (defined $@ and $@)
{
    die Dumper ($@);
}

##
# main();
# -------
#  Performs the installation
##
sub main
{
    # MKDoc install script beautiful ASCII headers
    print_hello();

    # check basic requirements
    create_mkdoc_sites_conf();
    check_perl();
    check_dbi();
    check_dbd_mysql();
    check_database_connect();
    
    # check for tidy
    $CONFIG->{TIDY_PATH} = check_tidy ($CONFIG->{TIDY_PATH});
    
    # check for HTML2TEXT dependencies
    check_html2text() if (defined $CONFIG->{H2TXT} and $CONFIG->{H2TXT});
    
    # check for RSS 090 Headlines dependencies
    check_r090hd() if (defined $CONFIG->{R090HD} and $CONFIG->{R090HD});
    
    # check for RSS 091 Headlines dependencies
    check_r091hd() if (defined $CONFIG->{R091HD} and $CONFIG->{R091HD});
    
    # check for RSS 100 Headlines dependencies
    check_r100hd() if (defined $CONFIG->{R100HD} and $CONFIG->{R100HD});
    
    # check for RSS 100 Sitemap dependencies
    check_r100sm() if (defined $CONFIG->{R100SM} and $CONFIG->{R100SM});
    
    # create needed directories and permissions
    setup_mkdoc_directories();
    
    # now we will write the configuration files from the configuration
    # templates, let's add a few extra keys to the $CONFIG hash
    my $site_dir = getcwd;
    chomp ($site_dir);
    $CONFIG->{SITE_DIR}  = $site_dir;
    write_config_files ($CONFIG);
    
    # check that the database connection works
    setup_database ($CONFIG);
    
    # now, we need to write the apache config file
    write_apache_config_file ('conf/httpd.conf.tmpl', $site_dir . '/httpd.conf');
    
    # finally, add this new site in the $mkdoc_dir/conf config files dir
    add_site_to_mkdoc();

    # create language templates from the generic templates
    create_templates();
        
    # tell the user what the password is for the admin interface and says bye-bye
    print_bye_bye();
}


##
# create_mkdoc_site_conf();
# -------------------------
#   The first time you create a site, the mkdoc/conf/sites.conf
#   file might not exist yet. This subroutine creates it if necessary.
##
sub create_mkdoc_sites_conf()
{
    my $mkdoc_dir = $CONFIG->{MKDOC_DIR};
    unless (-e "$mkdoc_dir/conf/sites.conf")
    {
        open FP, ">$mkdoc_dir/conf/sites.conf" or die "Cannot write open $mkdoc_dir/conf/sites.conf";
        print FP '$VAR1 = {};';
        close FP; 
    }
}


##
# print_bye_bye ($password);
# --------------------------
#   If we arrived at that stage, then the install script is nearly
#   done. Print a goodbye message with the password.
##
sub print_bye_bye
{
    print "\n";
    print "===============================================================\n";
    print "  The installation is now finished. You should make sure that  \n";
    print "  the following domain names are correctly set in your DNS:    \n";
    print "    + " . $CONFIG->{PUBLIC_DOMAIN} . "\n";
    print "    + " . $CONFIG->{ADMIN_DOMAIN} . "\n";
    print "    + " . $CONFIG->{SU_DOMAIN} . "\n";
    print "\n";
    print "  REMEMBER TO CREATE A .htaccess FILE TO PASSWORD PROTECT THE  \n";
    print "  SUPER USER INTERFACE !!!!!!                                  \n";
    print "\n";
    print "  You have to restart apache for the changes to take effect. \n\n";
    print "===============================================================\n";
}


##
# setup_database;
# ---------------
#   Creates def files and database tables
##
sub setup_database()
{
    my $site_dir = $CONFIG->{SITE_DIR};
    
    print "Setting up MKDoc database driver... ";
    open FP, ">$site_dir/su/driver.pl" or
	die "$site_dir/su/driver.pl";
    print FP "use lib::sql::Category;\n";
    print FP "use lib::sql::Table;\n";
    print FP "use lib::sql::type::Char;\n";
    print FP "use lib::sql::type::DateTime;\n";
    print FP "use lib::sql::type::Int;\n";
    print FP "use lib::sql::type::Text;\n";
    print FP "use lib::sql::type::LongText;\n";
    print FP "use lib::sql::DBH;\n";
    print FP "lib::sql::DBH->spawn (" . "\n";
    print FP "        database => '$CONFIG->{DATABASE}'," . "\n" if ($CONFIG->{DATABASE});
    print FP "        host     => '$CONFIG->{HOST}'," . "\n"     if ($CONFIG->{HOST});
    print FP "        port     => '$CONFIG->{PORT}'," . "\n"     if ($CONFIG->{PORT});
    print FP "        user     => '$CONFIG->{USER}'," . "\n"     if ($CONFIG->{USER});
    print FP "        password => '$CONFIG->{PASSWORD}'," . "\n" if ($CONFIG->{PASSWORD});
    print FP ");\n";
    print FP "lib::sql::Table->driver ('MySQL');"  . "\n";
    print FP "1;" . "\n";    
    print "OK\n";
    
    # try to eval the driver.pl to initialize the database
    print "Initializing database... ";
    
    open FP, "<su/driver.pl";
    my $data = join '', <FP>;
    close FP;
    eval $data;
    die $@ if ($@);
    
    # try to eval the schema.pl to instanciate the table objects
    open FP, "<schema.pl";
    my $data = join '', <FP>;
    close FP;
    eval $data;
    die $@ if ($@);
    
    # save the data def tables in the su directory
    lib::sql::Table->save_state ('su');

    # tries to silently drop everything
    try { lib::sql::Table->drop_all } catch {};
    
    # tries to create all the tables
    lib::sql::Table->create_all;
    
    # initialize database
    my $editor   = lib::sql::Table->table ('Editor');
    my $document = lib::sql::Table->table ('Document');
    my $base_document = lib::sql::Table->table ('Base_Document');
    
    $editor->insert ( Login       => 'admin',
                      Password    => 'admin',
                      Email       => $CONFIG->{ORGANIZATION_EMAIL},
                      First_Name  => '',
                      Family_Name => $CONFIG->{ORGANIZATION_NAME},
                      Disabled    => 1 );
    
    $editor->insert ( Login       => $CONFIG->{USER_LOGIN},
                      Password    => $CONFIG->{USER_PASSWORD},
                      Email       => $CONFIG->{USER_EMAIL},
                      First_Name  => $CONFIG->{USER_FIRST_NAME},
                      Family_Name => $CONFIG->{USER_LAST_NAME},
                      Disabled    => 0 );
    
    $base_document->insert ( Editor_ID => 2, Document_ID => 1 );
    
    $document->insert ( Template                => 'default',
                        Cache_Control           => 10,
                        Description             => 'enter a description here',
                        Keywords                => 'enter keywords here',
                        Date_Created            => current_date(),
                        Date_Last_Modified      => current_date(),
                        Editor_Created_ID       => 2,
                        Editor_Last_Modified_ID => 2,
                        Name                    => '',
                        Title                   => 'Root document',
                        Lang                    => 'en',
                        Sort_By                 => 'Title',
                        Order_By                => 0 );
    print "OK\n";
}


##
# current_date;
# -------------
#   Returns the current date in MySQL format
##
sub current_date
{
    # Prefill date fields with current date
    my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime (time);
    $mon++;
    $year += 1900;

    if (length ($mon)  == 1) { $mon  = "0" . $mon  }
    if (length ($mday) == 1) { $mday = "0" . $mday }

    return "$year-$mon-$mday $hour:$min:$sec";
}


##
# add_site_to_mkdoc;
# ------------------
#   Adds this website to the MKDoc sites config files and
#   amends conf/httpd.conf as well
##
sub add_site_to_mkdoc()
{
    my $mkdoc_dir = $CONFIG->{MKDOC_DIR};
    print "Re-building $mkdoc_dir/conf/sites.conf... ";
    
    # read conf/sites.conf data
    open FP, "<$mkdoc_dir/conf/sites.conf" or
	die "Cannot read-open $mkdoc_dir/conf/sites.conf";
    my $data = join '', <FP>;
    close FP;
    
    # instanciates the $VAR1 hashref
    my $VAR1 = undef;
    eval "$data";
    die $@ if (defined $@ and $@);
    
    # add the new site to the $VAR1 hashref
    $VAR1->{$CONFIG->{PUBLIC_DOMAIN}} = $CONFIG->{SITE_DIR};
    $VAR1->{$CONFIG->{ADMIN_DOMAIN}} = $CONFIG->{SITE_DIR};
    $VAR1->{$CONFIG->{SU_DOMAIN}} = $CONFIG->{SITE_DIR};
    
    # saves the $VAR1 hashref
    open FP, ">$mkdoc_dir/conf/sites.conf" or
	die "Cannot write-open $mkdoc_dir/conf/sites.conf";
    print FP Dumper ($VAR1);
    close FP;
    print "OK\n";
        
    # rewrites the conf/httpd.conf file
    print "Re-writing $mkdoc_dir/conf/httpd.conf... ";
    my $includes = { map { 'Include ' . $_ . '/httpd.conf' => 1 } values %{$VAR1} };
    open FP, ">$mkdoc_dir/conf/httpd.conf" or
	die "Cannot write-open $mkdoc_dir/conf/httpd.conf";
    for (keys %{$includes}) { print FP $_ . "\n" }
    close FP;
    print "OK\n";
}


##
# write_config_files;
# -------------------
#   Writes all the MKDoc config files for that site from what's
#   in $CONFIG
##
sub write_config_files()
{
    print "Writing config files...\n";
    die $@ if (defined $@ and $@);   
    write_config_file ('cgi.mkdoc');
    write_config_file ('cgi.admin');
    write_config_file ('flo.Editor');
    write_config_file ('flo.plugin.Cache')                    if (defined $CONFIG->{CACHE} and $CONFIG->{CACHE});
    write_config_file ('flo.plugin.Cache.Reader')             if (defined $CONFIG->{CACHE} and $CONFIG->{CACHE});
    write_config_file ('flo.plugin.Cache.Writer')             if (defined $CONFIG->{CACHE} and $CONFIG->{CACHE});
    write_config_file ('flo.plugin.Convert.TXT')              if (defined $CONFIG->{H2TXT} and $CONFIG->{H2TXT});
    write_config_file ('flo.plugin.Error');
    write_config_file ('flo.plugin.explorer.Admin');
    write_config_file ('flo.plugin.explorer.Admin_LANG');
    write_config_file ('flo.plugin.explorer.Print');
    write_config_file ('flo.plugin.explorer.Public');
    write_config_file ('flo.plugin.JavaScript.Headline');
    write_config_file ('flo.plugin.Search');
    write_config_file ('flo.plugin.Sitemap');
    write_config_file ('flo.plugin.Static');
    write_config_file ('flo.plugin.Poll');
    write_config_file ('flo.plugin.XML.IMS');
    write_config_file ('flo.plugin.XML.RDF.DC');
    write_config_file ('flo.plugin.XML.RDF.RSS.090.Headline') if (defined $CONFIG->{R090HD} and $CONFIG->{R090HD});
    write_config_file ('flo.plugin.XML.RSS.091.Headline')     if (defined $CONFIG->{R091HD} and $CONFIG->{R091HD});
    write_config_file ('flo.plugin.XML.RDF.RSS.100.Headline') if (defined $CONFIG->{R100HD} and $CONFIG->{R100HD});
    write_config_file ('flo.plugin.XML.RDF.RSS.100.Sitemap')  if (defined $CONFIG->{R100SM} and $CONFIG->{R100SM});
    write_config_file ('flo.plugin.Robots');
    write_config_file ('flo.plugin.Photo');
    write_config_file ('flo.XHTMLFilter');
    write_config_file ('mime.types');
    write_config_file ('GLOBAL');
    write_config_file ('flo.plugin.explorer.Admin_LANG_RTL');

    # this file can't be parsed by the template engine indeed
    write_template_config_file();
    print "OK\n";
}


##
# write_template_config_file;
# ---------------------------
#   The template config file cannot be written from a template config
#   because it defines the tags that the template engine uses itself,
#   therefore we have to hard code the content of this config file into
#   the install script
##
sub write_template_config_file()
{
    my $file = 'flo.Template';
    my $template_dir = $CONFIG->{SITE_DIR} . '/templates';
    my $data = <<'EOF';
LEFT_TOKEN      <%
RIGHT_TOKEN     %>
CODE_START      <!--#
CODE_END        #-->
COMMENT_START   <!--hide-->
COMMENT_END     <!--show-->

BASE_DIR        $template_dir/templates

# do not change these tags
sibling         flo::Template::Sibling
path            flo::Template::Path
include         flo::Template::Include
var             lib::Template::Var
if              lib::Template::If
perl            lib::Template::Perl
loop            flo::Template::Loop
document_list   flo::Template::DocumentList
browser_detect  flo::Template::BrowserDetect
uri_encode      flo::Template::URIEncode
a               flo::Template::Anchor
http            flo::Template::HTTP
http_date       flo::Template::HTTPDate
iso_date        flo::Template::ISODate
comment         flo::Template::Comment
link            flo::Template::Link
component       flo::Template::Component
to_html         flo::Template::Text2Html
EOF

    print "Writing $file... ";
    open FP, ">config/$file" or die "Cannot write-open config/$file";
    print FP qq |LEFT_TOKEN      <%|, "\n";
    print FP qq |RIGHT_TOKEN     %>|, "\n";
    print FP qq |BASE_DIR        $template_dir|,     "\n";

    print FP qq |sibling         flo::Template::Sibling|,      "\n";
    print FP qq |path            flo::Template::Path|,         "\n";
    print FP qq |include         flo::Template::Include|,      "\n";
    print FP qq |var             lib::Template::Var|,          "\n";
    print FP qq |if              lib::Template::If|,           "\n";
    print FP qq |perl            lib::Template::Perl|,         "\n";
    print FP qq |loop            flo::Template::Loop|,         "\n";
    print FP qq |document_list   flo::Template::DocumentList|, "\n";
    print FP qq |browser_detect  flo::Template::BrowserDetect|, "\n";
    print FP qq |uri_encode      flo::Template::URIEncode|, "\n";
    print FP qq |a               flo::Template::Anchor|, "\n";
    print FP qq |http            flo::Template::HTTP|, "\n";
    print FP qq |http_date       flo::Template::HTTPDate|, "\n";
    print FP qq |iso_date        flo::Template::ISODate|, "\n";
    print FP qq |comment         flo::Template::Comment|, "\n";
    print FP qq |link            flo::Template::Link|, "\n";
    print FP qq |component       flo::Template::Component|, "\n";
    print FP qq |to_html         flo::Template::Text2Html|, "\n";
    print FP qq |headlines       flo::Template::Headlines|, "\n";
    close FP;
    print "OK\n";
}


##
# write_config_file ($file);
# --------------------------
#   Writes the $CONFIG file using variables set in $CONFIG
##
sub write_config_file
{
    my $file = shift;

    # initialize template module
    $lib::Template::CONFIG = new lib::Config;
    $lib::Template::CONFIG->set (qw /LEFT_TOKEN <%/);
    $lib::Template::CONFIG->set (qw /RIGHT_TOKEN %>/);
    $lib::Template::CONFIG->set ('BASE_DIR', $CONFIG->{SITE_DIR});
    $lib::Template::CONFIG->set ('var', 'lib::Template::Var');
    $lib::Template::CONFIG->set ('if', 'lib::Template::If');
    $lib::Template::RIGHT_TOKEN = quotemeta ('%>');
    $lib::Template::LEFT_TOKEN  = quotemeta ('<%');
    $lib::Template::BASE_DIR    = $CONFIG->{SITE_DIR};
    $lib::Template::TOKEN_REGEX = $lib::Template::LEFT_TOKEN . '.*?' . $lib::Template::RIGHT_TOKEN;
    $lib::Template::STASH = {};
    my $tmpl = new lib::Template ( file => 'config_templates/' . $file, hash => $CONFIG );
    open FP, ">config/$file" or die "Cannot write-open config/$file";
    print FP $tmpl->parse;
    close FP;
    print " + $file\n";
}


##
# write_apache_config_file ($template, $file);
# --------------------------------------------
#   Writes the apache config file for that website
##
sub write_apache_config_file
{
    my $template = shift;
    my $file = shift;
    
    print "Writing $file... ";
    # initialize template module
    eval "use lib::Template";
    $lib::Template::CONFIG = new lib::Config;
    $lib::Template::CONFIG->set (qw /LEFT_TOKEN <%/);
    $lib::Template::CONFIG->set ('BASE_DIR', $CONFIG->{MKDOC_DIR});
    $lib::Template::CONFIG->set (qw /RIGHT_TOKEN %>/);
    $lib::Template::CONFIG->set ('var', 'lib::Template::Var');
    $lib::Template::CONFIG->set ('if', 'lib::Template::If');
    $lib::Template::RIGHT_TOKEN = quotemeta ('%>');
    $lib::Template::LEFT_TOKEN  = quotemeta ('<%');
    $lib::Template::BASE_DIR    = $CONFIG->{MKDOC_DIR};
    $lib::Template::TOKEN_REGEX = $lib::Template::LEFT_TOKEN . '.*?' . $lib::Template::RIGHT_TOKEN;
    $lib::Template::STASH = {};
    my $tmpl = new lib::Template ( file => $template, hash => $CONFIG );
    open FP, ">$file" or die "Cannot write-open $file";
    print FP $tmpl->parse;
    close FP;
    print "OK\n";
}


##
# setup_mkdoc_directories();
# --------------------------
#   This function is in charge of creating directories
#   which will be needed for this site and to give them
#   the right permissions
##
sub setup_mkdoc_directories()
{
    print "Setting up MKDoc directories... ";
    
    # if the cache dir is not there, let's create it
    # then let's change the permission
    unless (-e 'cache')
    {
	mkdir 'cache', 0777;
    }
    chmod 0777, 'cache';
    
    # if the tmp dir is not there, let's create it
    # then let's change the permission
    unless (-e 'tmp')
    {
	mkdir 'tmp', 0777;
    }
    chmod 0777,  'tmp';    
    
    # if the static dir is not here let's create it
    unless (-e 'static')
    {
	mkdir 'static', 0777;
    }
    
    # if the config dir is not here let's create it
    unless (-e 'config')
    {
	mkdir 'config', 0777;
    }
    
    # if the static/images dir is not here let's create it
    # then let's change the permission
    unless (-e 'static/images')
    {
	mkdir 'static/images', 0777;
    }
    chmod 0777, 'static/images';
    
    # if the static/images dir is not here let's create it
    # then let's change the permission
    unless (-e 'static/files')
    {
	mkdir 'static/files', 0777;
    }
    chmod 0777, 'static/files';
    
    # create the apache log directory
    unless (-e 'log')
    {
	mkdir 'log', 0777;
    }
    chmod 0777, 'log';
    
    # create the su directory
    unless (-e 'su')
    {
	mkdir 'su', 0777;
    }
    
    chmod 0700, 'install.conf';
    print "OK\n";
}


##
# check_database_connect;
# -----------------------
#   Attempts to connect to the MySQL database
##
sub check_database_connect()
{
    my $mysql_host     = $CONFIG->{HOST};
    my $mysql_database = $CONFIG->{DATABASE};
    my $mysql_port     = $CONFIG->{PORT};
    my $mysql_user     = $CONFIG->{USER};
    my $mysql_password = $CONFIG->{PASSWORD};
    
    print "Trying to connect to MySQL... ";
    eval
    {
	my $dsn = "DBI:mysql:database=$mysql_database";
	$dsn   .= ":host=$mysql_host" if (defined $mysql_host and $mysql_host);
        $dsn   .= ":port=$mysql_port" if (defined $mysql_port and $mysql_port);
	my $dbh = DBI->connect($dsn, $mysql_user, $mysql_password, {'RaiseError' => 1});
    };
    
    if ($@)
    {
	print "FAILED\n\n";
	print "$@";
	exit;
    }
    
    print "OK\n";
}


##
# check_r090hd();
# ---------------
#   Checks for RSS 090 Headlines plugin dependencies
#     + XML::RSS
##
sub check_r090hd
{
    print "Checking for RSS 090 Headlines plugin dependencies... ";
    eval "use XML::RSS";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need XML::RSS to use the RSS 090 Headlines plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of XML::RSS.\n";
	print "Installation aborted.\n";
	exit;
    }
    print "OK\n";
}


##
# check_r091hd();
# ---------------
#   Checks for RSS 091 Headlines plugin dependencies
#     + XML::RSS
##
sub check_r091hd
{
    print "Checking for RSS 091 Headlines plugin dependencies... ";
    eval "use XML::RSS";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need XML::RSS to use the RSS 091 Headlines plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of XML::RSS.\n";
	print "Installation aborted.\n";
	exit;
    }
    print "OK\n";
}


##
# check_r100hd();
# ---------------
#   Checks for RSS 100 Headlines plugin dependencies
#     + XML::RSS
##
sub check_r100hd
{
    print "Checking for RSS 100 Headlines plugin dependencies... ";
    eval "use XML::RSS";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need XML::RSS to use the RSS 100 Headlines plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of XML::RSS.\n";
	print "Installation aborted.\n";
	exit;
    }
    print "OK\n";
}


##
# check_r100sm();
# ---------------
#   Checks for RSS 100 Headlines plugin dependencies
#     + XML::RSS
##
sub check_r100sm
{
    print "Checking for RSS 100 Sitemap plugin dependencies... ";
    eval "use XML::RSS";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need XML::RSS to use the RSS 100 Sitemap plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of XML::RSS.\n";
	print "Installation aborted.\n";
	exit;
    }
    print "OK\n";
}


##
# check_html2text();
# ------------------
#   Checks for HTML to TEXT plugin dependencies
#     + HTML::TreeBuilder
#     + HTML::FormatText
##
sub check_html2text
{
    print "Checking for HTML to TEXT plugin dependencies... ";
    eval "use HTML::TreeBuilder";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need HTML::FormatText to use the HTML 2 TEXT plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of HTML::TreeBuilder.\n";
	print "Installation aborted.\n";
	exit;
    }
    
    eval "use HTML::FormatText";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need HTML::FormatText to use the HTML 2 TEXT plugin.\n";
	print "Check ftp:://ftp.cpan.org to download a recent version of HTML::FormatText.\n";
	print "Installation aborted.\n";
	exit;
    }

    print "OK\n";
}


##
# prompt_mkdoc_dir();
# -------------------
#   Prompts for the installation dir of MKDoc and returns it
##
sub prompt_mkdoc_dir()
{
    print "\n";
    print "Enter MKDoc 1.1 installation directory. [/opt/mkdoc]: ";
    my $mkdoc_dir = <STDIN>;
    chomp ($mkdoc_dir);
    $mkdoc_dir =~ s/\/$//;
    $mkdoc_dir ||= '/opt/mkdoc';
    unless (-e $mkdoc_dir and -d $mkdoc_dir)
    {
	print "\n";
	print "$mkdoc_dir does not exist or is not a valid directory!\n";
	print "Installation aborted.\n";
	exit;
    }
    
    return $mkdoc_dir;
}


##
# print_hello();
# --------------
#   Print a welcome / warning message and asks wether to continue
#   or not
##
sub print_hello()
{
    print "===============================================================\n";
    print "  MKDoc 1.1 website install script                             \n";
    print "===============================================================\n\n";
    print "  This installation script will attempt to setup a new MKDoc   \n";
    print "  website using the information provided in the install.conf   \n";
    print "  file.                                                        \n\n";
    print "  Please make sure that install.conf parameters are correct    \n";
    print "  before you continue.                                         \n\n";
    print "  ATTENTION! If the website which is set in install.conf does  \n";
    print "  exist, all its data will be removed.                         \n";
    print "===============================================================\n\n";
    print "Are you sure you want to continue [y|N] ?";
    my $in = <STDIN>;
    chomp ($in);
    unless ($in eq 'y' or $in eq 'Y')
    {
	print "Good bye!\n\n";
	exit;
    }
    print "\n";
}


##
# check_perl();
# -------------
#   Checks that we are using a version of Perl which is recent enough
##
sub check_perl()
{
    print "Checking for Perl 5.005_03... ";
    eval "use 5.005_03";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need at least Perl 5.005_03 to use this software.\n";
	print "Check http://www.perl.com to download a recent version of Perl.\n";
	print "Installation aborted.\n";
	exit;
    }
    else
    {
	print "OK\n";
    }
}


##
# check_dbi;
# ----------
#   Checks that the DBI module are installed
#   so that we can access the MySQL database
##
sub check_dbi()
{
    print "Checking for DBI... ";
    eval "use DBI";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need CPAN's DBI module to use this software.\n";
	print "Check ftp://ftp.cpan.org to download the latest version of this module.\n";
	print "Installation aborted.\n";
	exit;
    }
    else
    {
	print "OK\n";
    }
}


##
# check_dbd_mysql();
# ------------------
#   Checks that the Msql-Mysql DBD modules are installed
#   so that we can access the MySQL database
##
sub check_dbd_mysql()
{
    print "Checking for DBD::mysql... ";
    eval "use DBD::mysql";
    if ($@)
    {
	print "FAILED\n\n";
	print "You need CPAN's DBD msql-mysql modules to use this software.\n";
	print "Check ftp://ftp.cpan.org to download the latest version of this module.\n";
	print "Installation aborted.\n";
	exit;
    }
    else
    {
	print "OK\n";
    }
}


##
# check_tidy ($tidy_path);
# ------------------------
#   Checks that tidy is installed
##
sub check_tidy($)
{
    print "\n";
    print "Checking for HTML Tidy... ";
    my $tidy  = shift;
    unless (defined $tidy or $tidy or -e $tidy)
    {
        print "FAILED\n\n";
        print "You need Raggett's HTML Tidy program to use this software.\n";
        print "Check http://www.w3.org/People/Raggett/tidy to download the latest version of this program.\n";
        print "Installation aborted.\n";
    }
    print "OK\n";
    return $tidy;
}

sub create_templates
{
  my @args = (
    'perl',
      "$CONFIG->{MKDOC_DIR}/tools/mklang.pl",
      '--resourcepath=./resource',
      '--templatepath=./templates'
#      '--quiet'
  );

  system(@args) == 0
    or die "system @args failed: $?"
}

1;
